%=========================================================================================
% TUTORIAL 6 to 'Numerical Methods for PDEs', WS 01/02
% Lecture given by Dr.C.Coclici, assisted by M.Rheinlnder
%=========================================================================================
% This program illustrates how the Riemann problem is solved numerically.
% The left value a and the right value b are to be defined by the user; the
% jump is to be supposed at zero.
%
%
%=========================================================================================

disp('Choose the numerical schemes to be displayed (yes=1 no=0):');
LF = input('Lax-Friedrichs: ');
EO = input('Enquist-Osher : ');
G  = input('Godunov       : ');
U  = input('Upwind        : ');
a  = input('left  value a : ');
b  = input('right value b : ');

M    =   200;  %number of grid nodes (discretization points)
alfa =  -0.5;
beta =   1.5;
cn   =   0.7;
dx     = (beta - alfa)/M; 		
x      = [alfa+0.5*dx:dx:beta-0.5*dx];
n =  0;   %iteration-counter
t =  0;   %time
ns=  0;   %number of selected schemes
NS=['              ';'              ';'              ';'              '];

it=0;
IT=0;
ITmax=1000;

%--------------------------------------
% initialization
%--------------------------------------

v0 = a*(x<=0)+b*(x>=0);
dt = cn*dx/max(abs(df(v0)));
lambda = dt/dx;
vmin = min(a,b);
vmax = max(a,b);

if(LF==1) v_LF = v0;  ns=ns+1;  NS(ns,:) = 'Lax-Friedrichs'; end;
if(EO==1) v_EO = v0;  ns=ns+1;  NS(ns,:) = 'Enquist-Osher '; end;
if(G ==1) v_G  = v0;  ns=ns+1;  NS(ns,:) = 'Godunov       '; end;
if(U ==1) v_U  = v0;  ns=ns+1;  NS(ns,:) = 'Upwind        '; end;

%--------------------------------------
% while-loop
%--------------------------------------
while(1)
  figure(1);clf;hold on;
  if(LF==1) plot([alfa-dx/2 x beta+dx/2],[a v_LF b],'k-'); end;
  if(EO==1) plot([alfa-dx/2 x beta+dx/2],[a v_EO b],'b-'); end;
  if(G ==1) plot([alfa-dx/2 x beta+dx/2],[a v_G  b],'r--'); end;
  if(U ==1) plot([alfa-dx/2 x beta+dx/2],[a v_U  b],'m-'); end;
  title('The 1D Riemann Problem');
  text(alfa+0.03*(beta-alfa),vmax+0.1*(vmax-vmin)-0.02*1.2*(vmax-vmin),['iteration IT= ' int2str(IT) ]);
  legend(NS(1,:),NS(2,:),NS(3,:),NS(4,:),0);
  axis([alfa beta vmin-0.1*(vmax-vmin) vmax+0.1*(vmax-vmin)]);

  it = input('number of performing time-steps ','s');
  if(isempty(it)==1) it=1;
  elseif(it=='q') break; end;
  it = numeric(it);
  if(IT+it < 0) continue; end;
  if(IT+it > ITmax) it = ITmax - IT; end;
  t = t + it*dt;
  IT = IT + it;
  if(it>=0) n=it;
  else      n=IT;
    if(LF==1) v_LF = v0; end;
    if(EO==1) v_EO = v0; end;
    if(G ==1) v_G  = v0; end;
    if(U ==1) v_U  = v0; end;
  end;

  %there seems still to be sth. wrong with the boundary conditions
  for i=1:n,
    if(LF==1)%Lax-Friedrichs
      v_LF = v_LF - lambda * ( F_LF(lambda,v_LF,[v_LF(2:M) b]) - F_LF(lambda,[a v_LF(1:M-1)],v_LF) );
      %v_LF = v_LF - lambda * ( F_LF(lambda,v_LF,[v_LF(2:M) v_LF(M-1)]) - F_LF(lambda,[v_LF(2) v_LF(1:M-1)],v_LF) );
    end;
    if(EO==1)%Enquist-Osher
      v_EO = v_EO - lambda * ( F_EO(v_EO,[v_EO(2:M) b],0) - F_EO([a v_EO(1:M-1)],v_EO,0) );
    end;
    if(G ==1)%Godunov
      v_G  = v_G  - lambda * ( F_G (v_G ,[v_G(2:M)  b],0) - F_G ([a v_G(1:M-1) ],v_G ,0) );
    end;
      if(U ==1)%combined Upwind (not conservative)
      %v_U  = v_U  - lambda * ( (f(v_U) - f([v_U(M) v_U(1:M-1)])).*(df(v_U)>0) + (f([v_U(2:M) v_U(1)]) - f(v_U)).*(df(v_U)<0) );
      %v_U  = v_U  - lambda * ( F_LU (v_U ,[v_U(2:M)  v_U(1) ]) - F_LU ([v_U(M)  v_U(1:M-1) ],v_U ) );
      v_U   = v_U  - lambda * ( F_RU (v_U ,[v_U(2:M)  b]) - F_RU ([a  v_U(1:M-1) ],v_U ) );
    end;
  end;%for
end; %while